// CH552 Dragon (Ref. HW A031219)
//
// Touch key demo.
// If TIN3 is touched LED1 turns on, if TIN2 is touched LED2 turns on,
// if TIN1 is touched LED1 and LED 2 turn off.
// If the USR key is pressed the bootloader will be activated
// and the CH552 will be ready for a new flash programming operation.
//
// CREDITS
// Touch Key routines by Aaron Christophel
// http://atcnetz.blogspot.com/2019/02/ch552-020-mikrocontroller-mit-usb.html

#include <ch554.h>
#include <debug.h>
#include <bootloader.h>

// HW definitions (see A031219 schematic)
#define LED1_PIN 4		// LED1
#define LED2_PIN 3		// LED2
#define USR_KEY_PIN 2	// USR key (HIGH if not pressed)

SBIT(LED1, 0xB0, LED1_PIN);
SBIT(LED2, 0xB0, LED2_PIN);
SBIT(USR_KEY, 0xB0, USR_KEY_PIN);

//**********************************************
// Bootloader manual activation
//**********************************************

void jump_to_bootloader()
// Reset USB (if used) and disable all the interrupts, then jump to the bootloader
{
	USB_INT_EN = 0;		// Set USB_INT_EN to the reset value
	USB_CTRL = 0x06;	// Set USB_CTRL to the reset value
	EA = 0;				// Set the Global interrupt enable control bit to the reset value
	mDelaymS(100);
	bootloader();		// Jump to the bootloader
	while(1);
}

//**********************************************
// Touch Keys
//**********************************************

typedef unsigned char volatile          UINT8V;

#define		BIT0		(0X01)
#define		BIT1		(0X02)
#define		BIT2		(0X04)
#define		BIT3		(0X08)
#define		BIT4		(0X10)
#define		BIT5		(0X20)
#define		BIT6		(0X40)
#define		BIT7		(0X80)

#define		CHX				(0X00)
#define		CH1				(BIT1)				
#define		CH2				(BIT2)
#define		CH3				(BIT3)
#define		CH_FREE			(0x07)				

#define		TH_VALUE		(100)
#define		TOUCH_NUM		(0x04)
#define		SAMPLE_TIMES	(0x05)

__xdata uint8_t 	TK_Code[TOUCH_NUM] = {							
	0x03, 0x04,	0x02									
};		

__xdata uint16_t 	Key_FreeBuf[TOUCH_NUM];
__xdata UINT8V		Touch_IN;		

uint8_t TK_SelectChannel( uint8_t ch )
{
	if ( ch <= TOUCH_NUM )
	{
		TKEY_CTRL = ( TKEY_CTRL & 0XF8) | TK_Code[ch];
		return 1;
	}
	return	0;
}

uint8_t TK_Init( uint8_t channel)
{

__xdata	uint8_t 	i,j;
__xdata	uint16_t 	sum;
__xdata	uint16_t 	OverTime;
	
	P1_DIR_PU &= ~channel;
	P1_MOD_OC &= ~channel;
	TKEY_CTRL |= bTKC_2MS ;

	for ( i = 0; i < TOUCH_NUM; i++ )
	{
		sum = 0;
		j = SAMPLE_TIMES;
		TK_SelectChannel( i );
		while( j-- )
		{
			OverTime = 0;
			while( ( TKEY_CTRL & bTKC_IF ) == 0 )
			{
				if( ++OverTime == 0 )
				{
					return 0;
				}
			}
			sum += TKEY_DAT;												
		}
		Key_FreeBuf[i] = sum / SAMPLE_TIMES;
	}
	IE_TKEY = 1;    
	return 1;
}

void	TK_int_ISR( void ) __interrupt (INT_NO_TKEY)
{
__xdata	static uint8_t ch = 0;
__xdata	uint16_t KeyData;
	KeyData = TKEY_DAT;
	
	if( KeyData < ( Key_FreeBuf[ch] - TH_VALUE ) )
	{
		Touch_IN |=  1 << ( TK_Code[ch] - 1 );
	}
	if( ++ch >= TOUCH_NUM )
	{
		ch = 0;
	}	
	TK_SelectChannel( ch );
}

//**********************************************
// Main
//**********************************************

void main() {
	
	// CH552 init
    CfgFsys();
	mDelaymS(5);
	
	// LED init (Note: USR key doesn't need init as the reset values are ok with pull-up on)
	P3_MOD_OC = 0xE7;				// Set P3.3 and P3.4 as push-pull for LED1 and LED2
	LED1 = 0;
	LED2 = 0;
	
	// Touch keys init
	TK_Init(BIT1+BIT4+BIT5);		// Init TIN1 (P1.1), TIN2 (P1.4) and TIN3 (P1.5)
	TK_SelectChannel(0);
	EA = 1;    						// Activate global interrupt

    while (1) {
		
		if( Touch_IN != 0 )
		{
			if( Touch_IN & CH1 ) {LED1 = 0; LED2 = 0;}		// TIN1 pressed: all led off
			if( Touch_IN & CH2 ) LED2 = 1;					// TIN2 pressed: LED2 on
			if( Touch_IN & CH3 ) LED1 = 1;					// TIN3 pressed: LED1 on
			Touch_IN = 0;
		}
		
		if (!USR_KEY) jump_to_bootloader();					// Activate the bootloader is USR is pressed
    }
}
